@@ -131,12 +131,23 @@ path_exists(const char *path)
 
 /* multi-thread locking for trap_path users */
 pthread_mutex_t trap_path_lock = PTHREAD_MUTEX_INITIALIZER;
+static sigset_t trap_path_sig_restore;
 
 /* multi-thread locking for ioctls */
 pthread_mutex_t ioctl_lock = PTHREAD_MUTEX_INITIALIZER;
 
-#define TRAP_PATH_LOCK pthread_mutex_lock (&trap_path_lock)
-#define TRAP_PATH_UNLOCK pthread_mutex_unlock (&trap_path_lock)
+#define TRAP_PATH_LOCK \
+    do { \
+        sigset_t sig_set; \
+        sigfillset(&sig_set); \
+        pthread_sigmask(SIG_SETMASK, &sig_set, &trap_path_sig_restore); \
+        pthread_mutex_lock (&trap_path_lock); \
+    } while (0)
+#define TRAP_PATH_UNLOCK \
+    do { \
+        pthread_mutex_unlock (&trap_path_lock); \
+        pthread_sigmask(SIG_SETMASK, &trap_path_sig_restore, NULL); \
+    } while (0)
 
 #define IOCTL_LOCK pthread_mutex_lock (&ioctl_lock)
 #define IOCTL_UNLOCK pthread_mutex_unlock (&ioctl_lock)
@@ -1431,39 +1442,76 @@ int statx(int dirfd, const char *pathname, int flags, unsigned mask, struct stat
     return r;
 }
 
+static bool is_dir_or_contained(const char *path, const char *dir, const char *subdir)
+{
+    if (!path || !dir)
+	return false;
+
+    const ssize_t subdir_len = strlen(subdir);
+    const size_t dir_len = strlen(dir);
+
+    return (dir_len + subdir_len <= strlen(path) &&
+	    strncmp(path, dir, dir_len) == 0 &&
+	    strncmp(path + dir_len, subdir, subdir_len) == 0 &&
+	    (path[dir_len + subdir_len] == '\0' || path[dir_len + subdir_len] == '/'));
+}
+
+static bool is_fd_in_mock(int fd, const char *subdir)
+{
+    static char fdpath[PATH_MAX];
+    static char linkpath[PATH_MAX];
+    libc_func(readlink, ssize_t, const char*, char *, size_t);
+
+    snprintf(fdpath, sizeof fdpath, "/proc/self/fd/%i", fd);
+    int orig_errno = errno;
+    ssize_t linklen = _readlink(fdpath, linkpath, sizeof linkpath);
+    errno = orig_errno;
+    if (linklen < 0 || linklen >= sizeof linkpath) {
+	perror("umockdev: failed to map fd to a path");
+	return false;
+    }
+    linkpath[linklen] = '\0';
+
+    return is_dir_or_contained(linkpath, getenv("UMOCKDEV_DIR"), subdir);
+}
+
 #define WRAP_FSTATFS(suffix) \
 int fstatfs ## suffix(int fd, struct statfs ## suffix *buf)	\
 { \
     libc_func(fstatfs ## suffix, int, int, struct statfs ## suffix *buf); \
     int r = _fstatfs ## suffix(fd, buf);			\
-    if (r != 0)							\
-	return r;						\
-								\
-    static char fdpath[PATH_MAX];				\
-    static char linkpath[PATH_MAX];				\
-    snprintf(fdpath, sizeof fdpath, "/proc/self/fd/%i", fd);	\
-    ssize_t linklen = readlink(fdpath, linkpath, sizeof linkpath); \
-    if (linklen < 0) {						\
-	perror("umockdev: failed to map fd to a path");		\
-	return 0;						\
+    if (r == 0 && is_fd_in_mock (fd, "/sys")) {			\
+	DBG(DBG_PATH, "testbed wrapped fstatfs64 (%i) points into mocked /sys; adjusting f_type\n", fd); \
+	buf->f_type = SYSFS_MAGIC;				\
     }								\
-\
-    const char *prefix = getenv("UMOCKDEV_DIR");		\
-    if (prefix) {						\
-	size_t prefix_len = strlen(prefix);			\
-	if (prefix_len + 5 <= strlen(linkpath) &&		\
-		strncmp(prefix, linkpath, prefix_len) == 0 &&	\
-		strncmp(linkpath + prefix_len, "/sys/", 5) == 0) { \
-	    DBG(DBG_PATH, "testbed wrapped fstatfs64 (%i) points into mocked /sys; adjusting f_type\n", fd);	\
-	    buf->f_type = SYSFS_MAGIC;				\
-	}							\
-    }								\
-    return 0;							\
+    return r;							\
 }
 
 WRAP_FSTATFS();
 WRAP_FSTATFS(64);
 
+#define WRAP_STATFS(suffix) \
+int statfs ## suffix(const char *path, struct statfs ## suffix *buf) {	\
+    libc_func(statfs ## suffix, int, const char*, struct statfs ## suffix *buf); \
+    int r;								\
+    TRAP_PATH_LOCK;							\
+    const char *p = trap_path(path);					\
+    if (p == NULL || p == path) {					\
+	r = _statfs ## suffix(path, buf);				\
+	TRAP_PATH_UNLOCK;						\
+	return r;							\
+    } \
+    DBG(DBG_PATH, "testbed wrapped statfs" #suffix "(%s) -> %s\n", path, p); \
+    r = _statfs ## suffix(p, buf);					\
+    TRAP_PATH_UNLOCK;							\
+    if (r == 0 && is_dir_or_contained(path, "/sys", ""))		\
+	buf->f_type = SYSFS_MAGIC;					\
+    return r;								\
+}
+
+WRAP_STATFS();
+WRAP_STATFS(64);
+
 #endif
 
 int __open_2(const char *path, int flags);
